/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKBROADCASTABLE_H_
#define FANCYSOCKBROADCASTABLE_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

class String;
class Map;

namespace Sock {

/**
 * An interface for contexts, which accept broadcast calls.
 * This interface is the abstraction a server sees from a single client connection.
 */
class FANCYSOCK_API Broadcastable: public virtual Fancy::Object {
protected:
	Broadcastable();
	virtual ~Broadcastable();
public:
	typedef Fancy::ObjPtr<Broadcastable> ptr_t;

    /**
     * @param subSystem The subsystem name to which a oneway invocation will be broadcasted.
     * @param arg The argument to the broadcast.
     * @param metadata Metadata to pass to the call.
     */
	virtual void broadcast(const Fancy::ObjPtr<String> &subSystem,
			               const Fancy::ObjPtr<Fancy::Object> &arg,
			               const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;

};
}
}

#endif /* FANCYSOCKBROADCASTABLE_H_ */
