/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKCALLBACKINVOCATIONCONTEXT_H_
#define FANCYSOCKCALLBACKINVOCATIONCONTEXT_H_

#include <fancysockinvocationcontext.h>

namespace Fancy {

class UID;
class Map;

namespace Sock {

class Broadcastable;

/**
 * An invocation context, which allows callbacks to the originating invoker.
 */
class FANCYSOCK_API CallbackInvocationContext: public virtual Fancy::Sock::InvocationContext {
protected:
	CallbackInvocationContext();
	virtual ~CallbackInvocationContext();
public:
	typedef Fancy::ObjPtr<CallbackInvocationContext> ptr_t;

    /**
     * @param callbackUID The ID of the callback as returned by
     *             {@link RemoteInvoker#registerCallback(InvocationHandler)}.
     * @param arg The argument, which is passed to the {@link InvocationHandler}.
     * @param metadata The metadata passed to the {@link InvocationHandler}.
     * @return The result of the callback
     */
    virtual Fancy::Object::ptr_t callBack(const Fancy::ObjPtr<Fancy::UID> &callbackUID,
    		                              const Fancy::Object::ptr_t &arg,
      		                              const Fancy::ObjPtr<Fancy::Map> &metadata) = 0;

    /**
     * @return The broadcast context for initiating broadcasts to all connected clients.
     */
    virtual Fancy::ObjPtr<Broadcastable> getBroadcastable() const = 0;
};

}

}

#endif /* FANCYSOCKCALLBACKINVOCATIONCONTEXT_H_ */
