/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

/****************************************************************************
**
** Author: Wolfgang Glas / EV-i
**
** A basic email client implementation for transporting arbitrary documents.
**
*****************************************************************************/

#ifndef FANCYSOCK_EMAIL_TRANSPORT_H
#define FANCYSOCK_EMAIL_TRANSPORT_H

#include <fancysockresolveoptions.h>
#include <fancysocktransport.h>


namespace Fancy {
  namespace Sock {
    class Address;

    class FANCYSOCK_API EmailClientTransport : virtual public Transport
    {
    protected:
      EmailClientTransport();
      virtual ~EmailClientTransport();

    public:
      typedef Fancy::ObjPtr<EmailClientTransport> ptr_t;

      enum AuthenticationMechanism { None, Plain, CramMD5 };

      static ptr_t newInstance();

      virtual void setHelo (const std::string &h) = 0;
      virtual void setSender (const std::string &s) = 0;
      virtual void setAddressee (const std::string &addr) = 0;
      virtual void setRelay (const Fancy::ObjPtr<Address> &relay) = 0;
      virtual void setTimeout(int msec) = 0;
      virtual void setResolveOptions(const Fancy::ObjPtr<ResolveOptions>& resolveOptions) = 0;

      /**
       * Sets the authentication information for SMTP AUTH.
       *
       * Some mechanisms may not be implemented or not be supported by the server, so expect Exceptions.
       */
      virtual void setAuthentication(AuthenticationMechanism mechanism, const std::string &user, const std::string &password) = 0;

      virtual const std::string &getSender() const = 0;
      virtual const std::string &getAddressee() const = 0;
      virtual const std::string &getHelo() const = 0;
      virtual Fancy::ObjPtr<Address> getRelay() const = 0;
      virtual int                getTimeout() const = 0;
      virtual const Fancy::ObjPtr<ResolveOptions> getResolveOptions() const = 0;

    private:
      EmailClientTransport (const EmailClientTransport &);
      EmailClientTransport &operator=(const EmailClientTransport &);
    };
  }
}

#endif // ! FANCYSOCK_EMAIL_TRANSPORT_H
