/***********************************************************
 * $Id$
 *
 * FancySock HTTPD helpers
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCK_HTTPD_CONNECTION_HANDLER_H_
#define FANCYSOCK_HTTPD_CONNECTION_HANDLER_H_

//#include <fancysockhttpdrequesthandler.h>
#include <fancysock.h>

//#include <fancymap.h>
#include <fancyrunnable.h>

#include <string>

namespace Fancy {
class Map;

namespace Sock {
class Device;

namespace HttpD {
class HttpDRequestHandler;

/**
 * HttpDConnectionHandler provides a basic handler for connections to a HttpServerTransport.
 * It distributes the content work to a HttpDRequestHandler dependend on the URL path.
 */
class FANCYSOCK_API HttpDConnectionHandler : public virtual Fancy::Runnable {

public:
    typedef Fancy::ObjPtr<HttpDConnectionHandler> ptr_t;

    /**
     * Creates a new HttpDConnectionHandler.
     *
     * requestHandlerMap contains HttpDRequestHandlers.
     *
     * Search happens this way:
     * The relative URL path is stripped from query and anchor stuff, i.e. anything from a ? or #
     * If the requestHandlerMap contains a HttpDRequestHandler with this key, it is used.
     * Otherwise, one "directory level" is stripped from the URL path, until the "root" (/) is reached.
     *
     * For a request to
     *   http://myapp.mycompany.com/a/b/c?foo=bar
     * the keys search in the requestHandlerMap are:
     *   "/a/b/c"
     *   "/a/b"
     *   "/a"
     *   "/"
     *
     * Note that there is no search for "/a/b/" or "/a/".
     *
     * When using deep url path patterns, consider putting handlers in the map multiple times,
     * to avoid large numbers of map key misses.
     *
     * If no HttpDRequestHandler can be found, an exception is thrown.
     *
     * Usually "/" is mapped to a ResourceProvider serving static content from some directory in the file system.
     */
    static ptr_t newInstance(
    		const Fancy::ObjPtr<Fancy::Sock::Device> &device,
    		const Fancy::ObjPtr<Fancy::Map> & requestHandlerMap);

    virtual void run() = 0;

};


}}} // end namespaces

#endif /* FANCYSOCK_HTTPD_CONNECTION_HANDLER_H_ */
