/***********************************************************
 * $Id$
 *
 * FancySock HTTPD helpers
 *
 * Copyright (C) 2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCK_HTTPD_CONNECTION_HANDLER_FACTORY_H_
#define FANCYSOCK_HTTPD_CONNECTION_HANDLER_FACTORY_H_

#include <fancysock.h>
#include <fancysockconnectionhandler.h>
#include <fancysockhttpdrequesthandler.h>

#include <string>

namespace Fancy {
class Map;

namespace Sock {

namespace HttpD {
//class HttpDRequestHandler;


class FANCYSOCK_API HttpDConnectionHandlerFactory : public virtual Fancy::Sock::ConnectionHandlerFactory {

protected:
	virtual ~HttpDConnectionHandlerFactory();

public:
    typedef Fancy::ObjPtr<HttpDConnectionHandlerFactory> ptr_t;

    /**
     * Creates a new HttpDConnectionHandlerFactory.
     *
     * requestHandlerMap contains HttpDRequestHandlers, for passing to new HttpDConnectionHandler instances.
     *
     * For details about the requestHandlerMap see the documentation of HttpDConnectionHandler::newInstance().
     * Keys other than the root key "/" must not end with a '/', i.e. use "/someDir" but not "/someDir/".
     */
    static ptr_t newInstance(const Fancy::ObjPtr<Fancy::Map> & requestHandlerMap);

    /**
     * Returns a HttpDConnectionHandler to serve the fresh connection on the Sock::Device device.
     *
     * The requestHandlerMap passed to HttpDConnectionHandlerFactory::newInstance() earlier is passed to the HttpDConnectionHandler, too.
     */
    virtual Fancy::ObjPtr<Fancy::Runnable> newConnectionHandler(const Fancy::ObjPtr<Fancy::Sock::Device> &device) = 0;

};


}}} // end namespaces


#endif /* FANCYSOCK_HTTPD_CONNECTION_HANDLER_FACTORY_H_ */
