/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKREMOTINGCONNECTIONHANDLER_H_
#define FANCYSOCKREMOTINGCONNECTIONHANDLER_H_

#include <fancythread.h>
#include <fancysockbroadcastable.h>
#include <fancysockconnectionhandler.h>

namespace Fancy {

class SerializableFactory;
class String;

namespace Sock {

class Broadcastable;
class InvocationHandler;

class FANCYSOCK_API RemotingConnectionHandlerFactory:
                        public virtual ConnectionHandlerFactory,
                        public virtual Broadcastable {

protected:
	RemotingConnectionHandlerFactory();
	~RemotingConnectionHandlerFactory();
public:
	typedef Fancy::ObjPtr<RemotingConnectionHandlerFactory> ptr_t;

	static ptr_t newInstance(const Fancy::ObjPtr<Fancy::SerializableFactory> &serializableFactory);

	virtual void registerInvocationHandler(const Fancy::ObjPtr<Fancy::String> &subSystem, const Fancy::ObjPtr<InvocationHandler> &handler) = 0;

	virtual void deregisterInvocationHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) = 0;

	virtual Fancy::ObjPtr<InvocationHandler> getInvocationHandler(const Fancy::ObjPtr<Fancy::String> &subSystem) const = 0;

	virtual int getBufferSize() const = 0;
	virtual void setBufferSize(int n) = 0;
};

}

}

#endif /* FANCYSOCKREMOTINGCONNECTIONHANDLER_H_ */
