/***********************************************************
 * $Id$
 *
 * FancyLibs
 *
 * Copyright (C) 2009-2011 ev-i GmbH & IBCL e.U.
 * Copyright (C) 2011-2013 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYSOCKSTATEFUL_H_
#define FANCYSOCKSTATEFUL_H_

#include <fancysock.h>
#include <fancyobject.h>

namespace Fancy {

namespace Sock {

/**
 * this interface is implemented by classes, which have an internal state, which
 * may be reported to the outside in a thread-safe manner.
 */
class FANCYSOCK_API Stateful: public virtual Fancy::Object {

public:
	typedef Fancy::ObjPtr<Stateful> ptr_t;

    /**
      This virtual function returns a user-readable status report for daemons,
      servers and other stateful facilities.

      This function should be implemented in a thread-safe manner.

      @param os The output stream to write to.
    */
	virtual void printStatus(std::ostream &os) const = 0;
	// FIXME fancylibs4 make non-const

protected:
	Stateful();
	virtual ~Stateful();
};

}

}

#endif /* FANCYSOCKSTATEFUL_H_ */
